@@ -23,11 +23,35 @@
 #include <QtTest/QTest>
 #include "browserapplication.h"
 
+#ifndef QTRY_COMPARE
+
+#define __TRY_TIMEOUT__ 5000
+#define __TRY_STEP__    50
+
+#define __QTRY(__expression__, __functionToCall__) \
+    do { \
+        int __i = 0; \
+        while (!(__expression__) &&  __i < __TRY_TIMEOUT__) { \
+            QTest::qWait(__TRY_STEP__); \
+            __i += __TRY_STEP__; \
+        } \
+        __functionToCall__; \
+    } while(0)
+
+#define QTRY_COMPARE(__expression__, __expected__) \
+    __QTRY((__expression__ == __expected__), QCOMPARE(__expression__, __expected__));
+
+#define QTRY_VERIFY(__expression__) \
+    __QTRY(__expression__, QVERIFY(__expression__));
+
+#endif // QTRY_COMPARE
+
 #undef QTEST_MAIN
 
 #define QTEST_MAIN(TestObject) \
 int main(int argc, char *argv[]) \
 { \
+    Q_INIT_RESOURCE(htmls); \
     Q_INIT_RESOURCE(data); \
     BrowserApplication app(argc, argv); \
     TestObject tc; \
@@ -36,4 +60,3 @@ int main(int argc, char *argv[]) \
 
 #endif
 
-
