@@ -82,14 +82,16 @@
 
 #include <qdebug.h>
 
-#define BOOKMARKBAR "Bookmarks Bar"
-#define BOOKMARKMENU "Bookmarks Menu"
+#define BOOKMARKBAR QT_TRANSLATE_NOOP("BookmarksManager", "Bookmarks Bar")
+#define BOOKMARKMENU QT_TRANSLATE_NOOP("BookmarksManager", "Bookmarks Menu")
 
 BookmarksManager::BookmarksManager(QObject *parent)
     : QObject(parent)
     , m_loaded(false)
     , m_saveTimer(new AutoSaver(this))
     , m_bookmarkRootNode(0)
+    , m_toolbar(0)
+    , m_menu(0)
     , m_bookmarkModel(0)
 {
     connect(this, SIGNAL(entryAdded(BookmarkNode *)),
@@ -129,26 +131,23 @@ void BookmarksManager::load()
                "%3").arg(reader.lineNumber()).arg(reader.columnNumber()).arg(reader.errorString()));
     }
 
-    BookmarkNode *toolbar = 0;
-    BookmarkNode *menu = 0;
     QList<BookmarkNode*> others;
     for (int i = m_bookmarkRootNode->children().count() - 1; i >= 0; --i) {
         BookmarkNode *node = m_bookmarkRootNode->children().at(i);
         if (node->type() == BookmarkNode::Folder) {
             // Automatically convert
-            if (node->title == tr("Toolbar Bookmarks") && !toolbar) {
+            if ((node->title == tr("Toolbar Bookmarks")
+                 || node->title == QLatin1String(BOOKMARKBAR)) && !m_toolbar) {
                 node->title = tr(BOOKMARKBAR);
-            }
-            if (node->title == tr(BOOKMARKBAR) && !toolbar) {
-                toolbar = node;
+
+                m_toolbar = node;
             }
 
             // Automatically convert
-            if (node->title == tr("Menu") && !menu) {
+            if ((node->title == tr("Menu")
+                 || node->title == QLatin1String(BOOKMARKMENU)) && !m_menu) {
                 node->title = tr(BOOKMARKMENU);
-            }
-            if (node->title == tr(BOOKMARKMENU) && !menu) {
-                menu = node;
+                m_menu = node;
             }
         } else {
             others.append(node);
@@ -156,22 +155,22 @@ void BookmarksManager::load()
         m_bookmarkRootNode->remove(node);
     }
     Q_ASSERT(m_bookmarkRootNode->children().count() == 0);
-    if (!toolbar) {
-        toolbar = new BookmarkNode(BookmarkNode::Folder, m_bookmarkRootNode);
-        toolbar->title = tr(BOOKMARKBAR);
+    if (!m_toolbar) {
+        m_toolbar = new BookmarkNode(BookmarkNode::Folder, m_bookmarkRootNode);
+        m_toolbar->title = tr(BOOKMARKBAR);
     } else {
-        m_bookmarkRootNode->add(toolbar);
+        m_bookmarkRootNode->add(m_toolbar);
     }
 
-    if (!menu) {
-        menu = new BookmarkNode(BookmarkNode::Folder, m_bookmarkRootNode);
-        menu->title = tr(BOOKMARKMENU);
+    if (!m_menu) {
+        m_menu = new BookmarkNode(BookmarkNode::Folder, m_bookmarkRootNode);
+        m_menu->title = tr(BOOKMARKMENU);
     } else {
-        m_bookmarkRootNode->add(menu);
+        m_bookmarkRootNode->add(m_menu);
     }
 
     for (int i = 0; i < others.count(); ++i)
-        menu->add(others.at(i));
+        m_menu->add(others.at(i));
 }
 
 void BookmarksManager::save() const
@@ -182,8 +181,21 @@ void BookmarksManager::save() const
     XbelWriter writer;
     QString dir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
     QString bookmarkFile = dir + QLatin1String("/bookmarks.xbel");
+    // Save root folder titles in English (i.e. not localized)
+    m_menu->title = QLatin1String(BOOKMARKMENU);
+    m_toolbar->title = QLatin1String(BOOKMARKBAR);
     if (!writer.write(bookmarkFile, m_bookmarkRootNode))
         qWarning() << "BookmarkManager: error saving to" << bookmarkFile;
+    // Restore localized titles
+    retranslate();
+}
+
+void BookmarksManager::retranslate() const
+{
+    if (m_menu)
+        m_menu->title = tr(BOOKMARKMENU);
+    if (m_toolbar)
+        m_toolbar->title = tr(BOOKMARKBAR);
 }
 
 void BookmarksManager::addBookmark(BookmarkNode *parent, BookmarkNode *node, int row)
@@ -239,13 +251,8 @@ BookmarkNode *BookmarksManager::menu()
     if (!m_loaded)
         load();
 
-    for (int i = m_bookmarkRootNode->children().count() - 1; i >= 0; --i) {
-        BookmarkNode *node = m_bookmarkRootNode->children().at(i);
-        if (node->title == tr(BOOKMARKMENU))
-            return node;
-    }
-    Q_ASSERT(false);
-    return 0;
+    Q_ASSERT(m_menu);
+    return m_menu;
 }
 
 BookmarkNode *BookmarksManager::toolbar()
@@ -253,13 +260,8 @@ BookmarkNode *BookmarksManager::toolbar()
     if (!m_loaded)
         load();
 
-    for (int i = m_bookmarkRootNode->children().count() - 1; i >= 0; --i) {
-        BookmarkNode *node = m_bookmarkRootNode->children().at(i);
-        if (node->title == tr(BOOKMARKBAR))
-            return node;
-    }
-    Q_ASSERT(false);
-    return 0;
+    Q_ASSERT(m_toolbar);
+    return m_toolbar;
 }
 
 BookmarksModel *BookmarksManager::bookmarksModel()
@@ -592,7 +594,8 @@ QMimeData *BookmarksModel::mimeData(const QModelIndexList &indexes) const
     QMimeData *mimeData = new QMimeData();
     QByteArray data;
     QDataStream stream(&data, QIODevice::WriteOnly);
-    foreach (QModelIndex index, indexes) {
+    QList<QUrl> urls;
+    foreach (const QModelIndex &index, indexes) {
         if (index.column() != 0 || !index.isValid())
             continue;
         QByteArray encodedData;
@@ -602,8 +605,10 @@ QMimeData *BookmarksModel::mimeData(const QModelIndexList &indexes) const
         const BookmarkNode *parentNode = node(index);
         writer.write(&buffer, parentNode);
         stream << encodedData;
+        urls.append(index.data(BookmarksModel::UrlRole).toUrl());
     }
     mimeData->setData(MIMETYPE, data);
+    mimeData->setUrls(urls);
     return mimeData;
 }
 
@@ -707,8 +712,8 @@ bool AddBookmarkProxyModel::filterAcceptsRow(int source_row, const QModelIndex &
 
 AddBookmarkDialog::AddBookmarkDialog(const QString &url, const QString &title, QWidget *parent, BookmarksManager *bookmarkManager)
     : QDialog(parent)
-    , m_url(url)
     , m_bookmarksManager(bookmarkManager)
+    , m_proxyModel(0)
 {
     setWindowFlags(Qt::Sheet);
     if (!m_bookmarksManager)
@@ -733,17 +738,28 @@ AddBookmarkDialog::AddBookmarkDialog(const QString &url, const QString &title, Q
     view->setCurrentIndex(idx);
     location->setCurrentIndex(idx.row());
     name->setText(title);
+    address->setText(url);
+    if (!url.isEmpty())
+        address->hide();
+    address->setInactiveText(QLatin1String("Url"));
+    name->setInactiveText(QLatin1String("Title"));
+    resize(sizeHint());
 }
 
 void AddBookmarkDialog::accept()
 {
+    if (address->text().isEmpty() || name->text().isEmpty()) {
+        QDialog::accept();
+        return;
+    }
+
     QModelIndex index = location->view()->currentIndex();
     index = m_proxyModel->mapToSource(index);
     if (!index.isValid())
         index = m_bookmarksManager->bookmarksModel()->index(0, 0);
     BookmarkNode *parent = m_bookmarksManager->bookmarksModel()->node(index);
     BookmarkNode *bookmark = new BookmarkNode(BookmarkNode::Bookmark);
-    bookmark->url = m_url;
+    bookmark->url = address->text();
     bookmark->title = name->text();
     m_bookmarksManager->addBookmark(parent, bookmark);
     QDialog::accept();
@@ -752,6 +768,7 @@ void AddBookmarkDialog::accept()
 BookmarksMenu::BookmarksMenu(QWidget *parent)
     : ModelMenu(parent)
     , m_bookmarksManager(0)
+    , m_lastMouseButton(Qt::NoButton)
 {
     connect(this, SIGNAL(activated(const QModelIndex &)),
             this, SLOT(activated(const QModelIndex &)));
@@ -762,10 +779,16 @@ BookmarksMenu::BookmarksMenu(QWidget *parent)
 
 void BookmarksMenu::activated(const QModelIndex &index)
 {
-    emit openUrl(
-          index.data(BookmarksModel::UrlRole).toUrl(),
-          TabWidget::CurrentTab,
-          index.data(Qt::DisplayRole).toString());
+    if (m_lastMouseButton == Qt::MidButton)
+        emit openUrl(
+            index.data(BookmarksModel::UrlRole).toUrl(),
+            TabWidget::NewTab,
+            index.data(Qt::DisplayRole).toString());
+    else
+        emit openUrl(
+            index.data(BookmarksModel::UrlRole).toUrl(),
+            TabWidget::CurrentTab,
+            index.data(Qt::DisplayRole).toString());
 }
 
 bool BookmarksMenu::prePopulated()
@@ -782,6 +805,15 @@ bool BookmarksMenu::prePopulated()
     return true;
 }
 
+void BookmarksMenu::mouseReleaseEvent(QMouseEvent *event)
+{
+    m_lastMouseButton = event->button();
+    //alow normal processing
+    ModelMenu::mouseReleaseEvent(event);
+    //reset
+    m_lastMouseButton = Qt::NoButton;
+}
+
 void BookmarksMenu::setInitialActions(QList<QAction*> actions)
 {
     m_initialActions = actions;
@@ -791,6 +823,9 @@ void BookmarksMenu::setInitialActions(QList<QAction*> actions)
 
 BookmarksDialog::BookmarksDialog(QWidget *parent, BookmarksManager *manager)
     : QDialog(parent)
+    , m_bookmarksManager(0)
+    , m_bookmarksModel(0)
+    , m_proxyModel(0)
 {
     m_bookmarksManager = manager;
     if (!m_bookmarksManager)
@@ -799,10 +834,11 @@ BookmarksDialog::BookmarksDialog(QWidget *parent, BookmarksManager *manager)
 
     tree->setUniformRowHeights(true);
     tree->setSelectionBehavior(QAbstractItemView::SelectRows);
-    tree->setSelectionMode(QAbstractItemView::ContiguousSelection);
+    tree->setSelectionMode(QAbstractItemView::ExtendedSelection);
     tree->setTextElideMode(Qt::ElideMiddle);
     m_bookmarksModel = m_bookmarksManager->bookmarksModel();
     m_proxyModel = new TreeProxyModel(this);
+    m_proxyModel->setFilterKeyColumn(-1);
     connect(search, SIGNAL(textChanged(QString)),
             m_proxyModel, SLOT(setFilterFixedString(QString)));
     connect(removeButton, SIGNAL(clicked()), tree, SLOT(removeSelected()));
@@ -869,11 +905,20 @@ void BookmarksDialog::customContextMenuRequested(const QPoint &pos)
     QMenu menu;
     QModelIndex index = tree->indexAt(pos);
     index = index.sibling(index.row(), 0);
-    if (index.isValid() && !tree->model()->hasChildren(index)) {
+    QModelIndex sourceIndex = m_proxyModel->mapToSource(index);
+    const BookmarkNode *node = m_bookmarksModel->node(sourceIndex);
+    if (index.isValid() && node->type() != BookmarkNode::Folder) {
         menu.addAction(tr("Open"), this, SLOT(openInCurrentTab()));
         menu.addAction(tr("Open in New Tab"), this, SLOT(openInNewTab()));
         menu.addSeparator();
     }
+    menu.addSeparator();
+    QAction *renameAction = menu.addAction(tr("Edit Name"), this, SLOT(editName()));
+    renameAction->setEnabled(index.flags() & Qt::ItemIsEditable);
+    if (index.isValid() && node->type() != BookmarkNode::Folder) {
+        menu.addAction(tr("Edit Address"), this, SLOT(editAddress()));
+    }
+    menu.addSeparator();
     QAction *deleteAction = menu.addAction(tr("Delete"), tree, SLOT(removeSelected()));
     deleteAction->setEnabled(index.flags() & Qt::ItemIsDragEnabled);
     menu.exec(QCursor::pos());
@@ -882,7 +927,9 @@ void BookmarksDialog::customContextMenuRequested(const QPoint &pos)
 void BookmarksDialog::open(TabWidget::Tab tab)
 {
     QModelIndex index = tree->currentIndex();
-    if (!index.parent().isValid())
+    QModelIndex sourceIndex = m_proxyModel->mapToSource(index);
+    const BookmarkNode *node = m_bookmarksModel->node(sourceIndex);
+    if (!index.parent().isValid() || !node || node->type() == BookmarkNode::Folder)
         return;
     emit openUrl(
           index.sibling(index.row(), 1).data(BookmarksModel::UrlRole).toUrl(),
@@ -900,24 +947,44 @@ void BookmarksDialog::openInNewTab()
     open(TabWidget::NewTab);
 }
 
+void BookmarksDialog::editName()
+{
+    QModelIndex idx = tree->currentIndex();
+    tree->edit(idx);
+}
+
+void BookmarksDialog::editAddress()
+{
+    QModelIndex idx = tree->currentIndex();
+    idx = idx.sibling(idx.row(), 1);
+    tree->edit(idx);
+}
+
 void BookmarksDialog::newFolder()
 {
     QModelIndex currentIndex = tree->currentIndex();
     QModelIndex idx = currentIndex;
-    if (idx.isValid() && !idx.model()->hasChildren(idx))
+    QModelIndex sourceIndex = m_proxyModel->mapToSource(idx);
+    const BookmarkNode *sourceNode = m_bookmarksModel->node(sourceIndex);
+    int row = -1; // default: append new folder as last item in selected folder
+    if (sourceNode && sourceNode->type() != BookmarkNode::Folder) {
+        // if selected item is not a folder, add new folder to parent folder,
+        // but direcly below the selected item
         idx = idx.parent();
+        row = currentIndex.row() + 1;
+    }
     if (!idx.isValid())
-        idx = tree->rootIndex();
+        idx = m_proxyModel->index(1, 0); // Select Bookmarks menu as default
     idx = m_proxyModel->mapToSource(idx);
     BookmarkNode *parent = m_bookmarksManager->bookmarksModel()->node(idx);
     BookmarkNode *node = new BookmarkNode(BookmarkNode::Folder);
     node->title = tr("New Folder");
-    m_bookmarksManager->addBookmark(parent, node, currentIndex.row() + 1);
+    m_bookmarksManager->addBookmark(parent, node, row);
 }
 
-BookmarkToolButton::BookmarkToolButton(QUrl url, QWidget *parent)
+BookmarkToolButton::BookmarkToolButton(const QModelIndex &index, QWidget *parent)
     : QToolButton(parent)
-    , m_url(url)
+    , m_index(index)
 {
 }
 
@@ -937,15 +1004,24 @@ void BookmarkToolButton::mouseReleaseEvent(QMouseEvent *event)
     }
 }
 
+QModelIndex BookmarkToolButton::index() const
+{
+    return m_index;
+}
+
 QUrl BookmarkToolButton::url() const
 {
-    return m_url;
+    return m_index.data(BookmarksModel::UrlRole).toUrl();
 }
 
 BookmarksToolBar::BookmarksToolBar(BookmarksModel *model, QWidget *parent)
     : QToolBar(tr("Bookmark"), parent)
     , m_bookmarksModel(model)
 {
+    setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(this, SIGNAL(customContextMenuRequested(const QPoint &)),
+            this, SLOT(contextMenuRequested(const QPoint &)));
+
     setRootIndex(model->index(0, 0));
     connect(m_bookmarksModel, SIGNAL(modelReset()), this, SLOT(build()));
     connect(m_bookmarksModel, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SLOT(build()));
@@ -954,6 +1030,101 @@ BookmarksToolBar::BookmarksToolBar(BookmarksModel *model, QWidget *parent)
     setAcceptDrops(true);
 }
 
+Q_DECLARE_METATYPE(QModelIndex)
+void BookmarksToolBar::contextMenuRequested(const QPoint &position)
+{
+    QToolButton *button = qobject_cast<QToolButton *>(childAt(position));
+
+    QMenu menu;
+    QAction *action;
+
+    if (button) {
+        QModelIndex index;
+        QVariant variant;
+
+        BookmarkToolButton *bookmarkButton = qobject_cast<BookmarkToolButton *>(button);
+        ModelMenu *modelMenu = qobject_cast<ModelMenu *>(button->menu());
+        if (modelMenu) {
+            index = modelMenu->rootIndex();
+            variant.setValue(index);
+        } else if (bookmarkButton) {
+            index = bookmarkButton->index();
+            variant.setValue(index);
+
+            action = menu.addAction(tr("Open"), this, SLOT(openBookmark()));
+            action->setData(variant);
+
+            action = menu.addAction(tr("Open in New &Tab"), this, SLOT(openBookmarkInNewTab()));
+            action->setData(variant);
+
+            menu.addSeparator();
+        }
+
+        if (variant.isValid()) {
+            action = menu.addAction(tr("Remove"), this, SLOT(removeBookmark()));
+            action->setData(variant);
+        }
+
+        menu.addSeparator();
+    }
+
+    action = menu.addAction(tr("Add Bookmark..."), this, SLOT(newBookmark()));
+
+    menu.exec(QCursor::pos());
+}
+
+void BookmarksToolBar::openBookmark()
+{
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (!action)
+        return;
+    QVariant variant = action->data();
+    if (!variant.canConvert<QModelIndex>())
+        return;
+    QModelIndex index = qvariant_cast<QModelIndex>(variant);
+
+    emit openUrl(
+        index.data(BookmarksModel::UrlRole).toUrl(),
+        TabWidget::CurrentTab,
+        index.data(Qt::DisplayRole).toString());
+}
+
+void BookmarksToolBar::openBookmarkInNewTab()
+{
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (!action)
+        return;
+    QVariant variant = action->data();
+    if (!variant.canConvert<QModelIndex>())
+        return;
+    QModelIndex index = qvariant_cast<QModelIndex>(variant);
+
+    emit openUrl(
+        index.data(BookmarksModel::UrlRole).toUrl(),
+        TabWidget::NewTab,
+        index.data(Qt::DisplayRole).toString());
+}
+
+void BookmarksToolBar::removeBookmark()
+{
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (!action)
+        return;
+    QVariant variant = action->data();
+    if (!variant.canConvert<QModelIndex>())
+        return;
+    QModelIndex index = qvariant_cast<QModelIndex>(variant);
+
+    m_bookmarksModel->removeRow(index.row(), m_root);
+}
+
+void BookmarksToolBar::newBookmark()
+{
+    AddBookmarkDialog *dialog = new AddBookmarkDialog(QString(), QString());
+    dialog->exec();
+    delete dialog;
+}
+
 void BookmarksToolBar::dragEnterEvent(QDragEnterEvent *event)
 {
     const QMimeData *mimeData = event->mimeData();
@@ -967,27 +1138,31 @@ void BookmarksToolBar::dropEvent(QDropEvent *event)
     const QMimeData *mimeData = event->mimeData();
     if (mimeData->hasUrls() && mimeData->hasText()) {
         QList<QUrl> urls = mimeData->urls();
-        QAction *action = actionAt(event->pos());
-        QString dropText;
-        if (action)
-            dropText = action->text();
+        QString title = mimeData->text();
+        QUrl url = urls.at(0);
+
+        if (url.isEmpty()) {
+            QToolBar::dropEvent(event);
+            return;
+        }
+
+        if (title.isEmpty()) {
+            title = QLatin1String(url.toEncoded());
+        }
+
         int row = -1;
         QModelIndex parentIndex = m_root;
-        for (int i = 0; i < m_bookmarksModel->rowCount(m_root); ++i) {
-            QModelIndex idx = m_bookmarksModel->index(i, 0, m_root);
-            QString title = idx.data().toString();
-            if (title == dropText) {
-                row = i;
-                if (m_bookmarksModel->hasChildren(idx)) {
-                    parentIndex = idx;
-                    row = -1;
-                }
-                break;
-            }
+        QToolButton *target = qobject_cast<QToolButton *>(childAt(event->pos()));
+
+        if (target && target->menu()) {
+            ModelMenu *menu = qobject_cast<ModelMenu *>(target->menu());
+            if (menu)
+                parentIndex = menu->rootIndex();
         }
+
         BookmarkNode *bookmark = new BookmarkNode(BookmarkNode::Bookmark);
-        bookmark->url = urls.at(0).toString();
-        bookmark->title = mimeData->text();
+        bookmark->url = QLatin1String(url.toEncoded());
+        bookmark->title = title;
 
         BookmarkNode *parent = m_bookmarksModel->node(parentIndex);
         BookmarksManager *bookmarksManager = m_bookmarksModel->bookmarksManager();
@@ -1009,6 +1184,30 @@ QModelIndex BookmarksToolBar::rootIndex() const
     return m_root;
 }
 
+BookmarksToolBarMenu::BookmarksToolBarMenu(QWidget *parent)
+    : ModelMenu(parent)
+    , m_lastMouseButton(Qt::NoButton)
+{
+    connect(this, SIGNAL(activated(const QModelIndex &)),
+            this, SLOT(activated(const QModelIndex &)));
+    setStatusBarTextRole(BookmarksModel::UrlStringRole);
+    setSeparatorRole(BookmarksModel::SeparatorRole);
+}
+
+void BookmarksToolBarMenu::activated(const QModelIndex &index)
+{
+    if (m_lastMouseButton == Qt::MidButton)
+        emit openUrl(
+            index.data(BookmarksModel::UrlRole).toUrl(),
+            TabWidget::NewTab,
+            index.data(Qt::DisplayRole).toString());
+    else
+        emit openUrl(
+            index.data(BookmarksModel::UrlRole).toUrl(),
+            TabWidget::CurrentTab,
+            index.data(Qt::DisplayRole).toString());
+}
+
 void BookmarksToolBar::build()
 {
     clear();
@@ -1019,7 +1218,8 @@ void BookmarksToolBar::build()
             button->setPopupMode(QToolButton::InstantPopup);
             button->setArrowType(Qt::DownArrow);
             button->setText(idx.data().toString());
-            ModelMenu *menu = new ModelMenu(this);
+            ModelMenu *menu = new BookmarksToolBarMenu(this);
+            menu->setMaxRows(-1);
             menu->setModel(m_bookmarksModel);
             menu->setRootIndex(idx);
             menu->addAction(new QAction(menu));
@@ -1027,9 +1227,10 @@ void BookmarksToolBar::build()
             button->setToolButtonStyle(Qt::ToolButtonTextOnly);
             QAction *a = addWidget(button);
             a->setText(idx.data().toString());
+            connect(menu, SIGNAL(openUrl(const QUrl &, TabWidget::Tab, const QString &)),
+                    this, SIGNAL(openUrl(const QUrl &, TabWidget::Tab, const QString &)));
         } else {
-            QUrl url = idx.data(BookmarksModel::UrlRole).toUrl();
-            BookmarkToolButton *button = new BookmarkToolButton(url, this);
+            BookmarkToolButton *button = new BookmarkToolButton(idx, this);
             button->setText(idx.data().toString());
             connect(button, SIGNAL(openBookmark(const QUrl &, TabWidget::Tab, const QString &)),
                     this, SIGNAL(openUrl(const QUrl &, TabWidget::Tab, const QString &)));
@@ -1038,3 +1239,11 @@ void BookmarksToolBar::build()
     }
 }
 
+void BookmarksToolBarMenu::mouseReleaseEvent(QMouseEvent *ev)
+{
+    m_lastMouseButton = ev->button();
+    //alow normal processing
+    ModelMenu::mouseReleaseEvent(ev);
+    //reset
+    m_lastMouseButton = Qt::NoButton;
+}
