@@ -96,6 +96,7 @@ public:
     void setTitle(BookmarkNode *node, const QString &newTitle);
     void setUrl(BookmarkNode *node, const QString &newUrl);
     void changeExpanded();
+    void retranslate() const;
 
     BookmarkNode *bookmarks();
     BookmarkNode *menu();
@@ -119,6 +120,8 @@ private:
     bool m_loaded;
     AutoSaver *m_saveTimer;
     BookmarkNode *m_bookmarkRootNode;
+    BookmarkNode *m_toolbar;
+    BookmarkNode *m_menu;
     BookmarksModel *m_bookmarkModel;
     QUndoStack m_commands;
 
@@ -205,8 +208,8 @@ public:
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     int columnCount(const QModelIndex &parent = QModelIndex()) const;
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    QModelIndex index(int, int, const QModelIndex& = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex& index = QModelIndex()) const;
+    QModelIndex index(int, int, const QModelIndex &parent = QModelIndex()) const;
+    QModelIndex parent(const QModelIndex &index = QModelIndex()) const;
     Qt::ItemFlags flags(const QModelIndex &index) const;
     Qt::DropActions supportedDropActions() const;
     bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex());
@@ -242,12 +245,16 @@ public:
 protected:
     bool prePopulated();
 
+protected:
+    virtual void mouseReleaseEvent(QMouseEvent *event);
+
 private slots:
     void activated(const QModelIndex &index);
 
 private:
     BookmarksManager *m_bookmarksManager;
     QList<QAction*> m_initialActions;
+    Qt::MouseButton m_lastMouseButton;
 };
 
 /*
@@ -281,7 +288,6 @@ private slots:
     void accept();
 
 private:
-    QString m_url;
     BookmarksManager *m_bookmarksManager;
     AddBookmarkProxyModel *m_proxyModel;
 };
@@ -304,6 +310,8 @@ private slots:
     void open(TabWidget::Tab tab);
     void openInNewTab();
     void openInCurrentTab();
+    void editName();
+    void editAddress();
     void newFolder();
 
 private:
@@ -323,18 +331,40 @@ signals:
     void openBookmark(const QUrl &url, TabWidget::Tab tab, const QString &title);
 
 public:
-    BookmarkToolButton(QUrl url, QWidget *parent = 0);
+    BookmarkToolButton(const QModelIndex &index, QWidget *parent = 0);
+    QModelIndex index() const;
     QUrl url() const;
 
 protected:
     void mouseReleaseEvent(QMouseEvent *event);
 
 private:
-    QUrl m_url;
+    QModelIndex m_index;
 
 };
 
 
+class BookmarksToolBarMenu : public ModelMenu
+{
+    Q_OBJECT
+
+signals:
+    void openUrl(const QUrl &url, TabWidget::Tab type, const QString &title);
+
+public:
+    BookmarksToolBarMenu(QWidget *parent = 0);
+
+protected:
+    virtual void mouseReleaseEvent(QMouseEvent *ev);
+
+private slots:
+    void activated(const QModelIndex &index);
+
+private:
+    Qt::MouseButton m_lastMouseButton;
+};
+
+
 class BookmarksToolBar : public QToolBar
 {
     Q_OBJECT
@@ -354,6 +384,13 @@ protected:
 private slots:
     void build();
 
+protected slots:
+    void contextMenuRequested(const QPoint &position);
+    void openBookmark();
+    void openBookmarkInNewTab();
+    void removeBookmark();
+    void newBookmark();
+
 private:
     BookmarksModel *m_bookmarksModel;
     QPersistentModelIndex m_root;
