@@ -67,6 +67,7 @@
 #include "cookiejar.h"
 #include "downloadmanager.h"
 #include "history.h"
+#include "languagemanager.h"
 #include "networkaccessmanager.h"
 #include "tabwidget.h"
 #include "webview.h"
@@ -78,13 +79,6 @@
 #include <qlibraryinfo.h>
 #include <qmessagebox.h>
 #include <qsettings.h>
-#include <qtextstream.h>
-#include <qtranslator.h>
-
-#include <qlocalserver.h>
-#include <qlocalsocket.h>
-#include <qnetworkproxy.h>
-
 #include <qwebsettings.h>
 
 #include <qdebug.h>
@@ -93,15 +87,15 @@ DownloadManager *BrowserApplication::s_downloadManager = 0;
 HistoryManager *BrowserApplication::s_historyManager = 0;
 NetworkAccessManager *BrowserApplication::s_networkAccessManager = 0;
 BookmarksManager *BrowserApplication::s_bookmarksManager = 0;
+LanguageManager *BrowserApplication::s_languageManager = 0;
 
 BrowserApplication::BrowserApplication(int &argc, char **argv)
-    : QApplication(argc, argv)
-    , m_localServer(0)
+    : SingleApplication(argc, argv)
     , quiting(false)
 {
     QCoreApplication::setOrganizationDomain(QLatin1String("arora-browser.org"));
     QCoreApplication::setApplicationName(QLatin1String("Arora"));
-    QString version = QLatin1String("0.4");
+    QString version = QLatin1String("0.5");
     if (QLatin1String(GITCHANGENUMBER) != QLatin1String("0"))
         version += QString(tr(" (Change: %1 %2)"))
                     .arg(QLatin1String(GITCHANGENUMBER))
@@ -109,39 +103,16 @@ BrowserApplication::BrowserApplication(int &argc, char **argv)
 
     QCoreApplication::setApplicationVersion(version);
 #ifndef AUTOTESTS
-#ifdef Q_WS_QWS
-    // Use a different server name for QWS so we can run an X11
-    // browser and a QWS browser in parallel on the same machine for
-    // debugging
-    QString serverName = QCoreApplication::applicationName() + QLatin1String("_qws");
-#else
-    QString serverName = QCoreApplication::applicationName();
-#endif // Q_WS_QWS
-    QLocalSocket socket;
-    socket.connectToServer(serverName);
-    if (socket.waitForConnected(500)) {
-        QTextStream stream(&socket);
-        QStringList args = QCoreApplication::arguments();
-        if (args.count() > 1)
-            stream << args.last();
-        else
-            stream << QString();
-        stream.flush();
-        socket.waitForBytesWritten();
+    QStringList args = QCoreApplication::arguments();
+    QString message = (args.count() > 1) ? args.last() : QString();
+    if (sendMessage(message))
         return;
-    }
-
-    m_localServer = new QLocalServer(this);
-    connect(m_localServer, SIGNAL(newConnection()),
-            this, SLOT(newLocalSocketConnection()));
-    if (!m_localServer->listen(serverName)) {
-        if (m_localServer->serverError() == QAbstractSocket::AddressInUseError
-            && QFile::exists(m_localServer->serverName())) {
-            QFile::remove(m_localServer->serverName());
-            m_localServer->listen(serverName);
-        }
-    }
-#endif // AUTOTESTS
+    // not sure what else to do...
+    if (!startSingleServer())
+        return;
+    connect(this, SIGNAL(messageRecieved(const QString &)),
+            this, SLOT(messageRecieved(const QString &)));
+#endif
 
 #if defined(Q_WS_MAC)
     QApplication::setQuitOnLastWindowClosed(false);
@@ -150,10 +121,6 @@ BrowserApplication::BrowserApplication(int &argc, char **argv)
 #endif
 
     QDesktopServices::setUrlHandler(QLatin1String("http"), this, "openUrl");
-    QString localSysName = QLocale::system().name();
-
-    installTranslator(QLatin1String("qt_") + localSysName);
-    installTranslator(dataDirectory() + QDir::separator() + QLatin1String("locale") + QDir::separator() + localSysName);
 
     // Until QtWebkit defaults to 16
     QWebSettings::globalSettings()->setFontSize(QWebSettings::DefaultFontSize, 16);
@@ -171,7 +138,8 @@ BrowserApplication::BrowserApplication(int &argc, char **argv)
 
 #ifndef AUTOTESTS
     QTimer::singleShot(0, this, SLOT(postLaunch()));
-#endif // AUTOTESTS
+#endif
+    LanguageManager();
 }
 
 BrowserApplication::~BrowserApplication()
@@ -198,6 +166,24 @@ BrowserApplication *BrowserApplication::instance()
     return (static_cast<BrowserApplication *>(QCoreApplication::instance()));
 }
 
+void BrowserApplication::messageRecieved(const QString &message)
+{
+    QUrl url(message);
+    if (!url.isEmpty()) {
+        QSettings settings;
+        settings.beginGroup(QLatin1String("general"));
+        int openLinksIn = settings.value(QLatin1String("openLinksIn"), 0).toInt();
+        settings.endGroup();
+        if (openLinksIn == 1)
+            newMainWindow();
+        else
+            mainWindow()->tabWidget()->newTab();
+        openUrl(url);
+    }
+    mainWindow()->raise();
+    mainWindow()->activateWindow();
+}
+
 #if defined(Q_WS_MAC)
 #include <qmessagebox.h>
 void BrowserApplication::quitBrowser()
@@ -410,20 +396,8 @@ bool BrowserApplication::restoreLastSession()
     return true;
 }
 
-bool BrowserApplication::isTheOnlyBrowser() const
-{
-    return (m_localServer != 0);
-}
-
-void BrowserApplication::installTranslator(const QString &name)
-{
-    QTranslator *translator = new QTranslator(this);
-    translator->load(name, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
-    QApplication::installTranslator(translator);
-}
-
 #if defined(Q_WS_MAC)
-bool BrowserApplication::event(QEvent* event)
+bool BrowserApplication::event(QEvent *event)
 {
     switch (event->type()) {
     case QEvent::ApplicationActivate: {
@@ -457,6 +431,8 @@ BrowserMainWindow *BrowserApplication::newMainWindow()
 {
     BrowserMainWindow *browser = new BrowserMainWindow();
     m_mainWindows.prepend(browser);
+    connect(this, SIGNAL(privacyChanged(bool)),
+            browser, SLOT(slotPrivacyChanged(bool)));
     browser->show();
     return browser;
 }
@@ -469,31 +445,6 @@ BrowserMainWindow *BrowserApplication::mainWindow()
     return m_mainWindows[0];
 }
 
-void BrowserApplication::newLocalSocketConnection()
-{
-    QLocalSocket *socket = m_localServer->nextPendingConnection();
-    if (!socket)
-        return;
-    socket->waitForReadyRead(1000);
-    QTextStream stream(socket);
-    QString url;
-    stream >> url;
-    if (!url.isEmpty()) {
-        QSettings settings;
-        settings.beginGroup(QLatin1String("general"));
-        int openLinksIn = settings.value(QLatin1String("openLinksIn"), 0).toInt();
-        settings.endGroup();
-        if (openLinksIn == 1)
-            newMainWindow();
-        else
-            mainWindow()->tabWidget()->newTab();
-        openUrl(url);
-    }
-    delete socket;
-    mainWindow()->raise();
-    mainWindow()->activateWindow();
-}
-
 CookieJar *BrowserApplication::cookieJar()
 {
     return (CookieJar*)networkAccessManager()->cookieJar();
@@ -501,9 +452,8 @@ CookieJar *BrowserApplication::cookieJar()
 
 DownloadManager *BrowserApplication::downloadManager()
 {
-    if (!s_downloadManager) {
+    if (!s_downloadManager)
         s_downloadManager = new DownloadManager();
-    }
     return s_downloadManager;
 }
 
@@ -525,12 +475,18 @@ HistoryManager *BrowserApplication::historyManager()
 
 BookmarksManager *BrowserApplication::bookmarksManager()
 {
-    if (!s_bookmarksManager) {
+    if (!s_bookmarksManager)
         s_bookmarksManager = new BookmarksManager;
-    }
     return s_bookmarksManager;
 }
 
+LanguageManager *BrowserApplication::languageManager()
+{
+    if (!s_languageManager)
+        s_languageManager = new LanguageManager;
+    return s_languageManager;
+}
+
 QIcon BrowserApplication::icon(const QUrl &url)
 {
     QIcon icon = QWebSettings::iconForUrl(url);
@@ -547,11 +503,22 @@ QIcon BrowserApplication::icon(const QUrl &url)
     return icon;
 }
 
-QString BrowserApplication::dataDirectory() const
+QString BrowserApplication::dataDirectory()
 {
 #if defined(Q_WS_X11)
     return QLatin1String(PKGDATADIR);
 #else
-    return applicationDirPath();
+    return qApp->applicationDirPath();
 #endif
 }
+
+bool BrowserApplication::isPrivate()
+{
+    return QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled);
+}
+
+void BrowserApplication::setPrivate(bool isPrivate)
+{
+    QWebSettings::globalSettings()->setAttribute(QWebSettings::PrivateBrowsingEnabled, isPrivate);
+    emit instance()->privacyChanged(isPrivate);
+}
