@@ -71,7 +71,11 @@
 #include "clearprivatedata.h"
 #include "downloadmanager.h"
 #include "history.h"
+#include "languagemanager.h"
+#include "networkaccessmanager.h"
+#include "networkmonitor.h"
 #include "settings.h"
+#include "sourceviewer.h"
 #include "tabbar.h"
 #include "tabwidget.h"
 #include "toolbarsearch.h"
@@ -79,8 +83,8 @@
 #include "webviewsearch.h"
 
 #include <qdesktopwidget.h>
+#include <qevent.h>
 #include <qfiledialog.h>
-#include <qplaintextedit.h>
 #include <qprintdialog.h>
 #include <qprintpreviewdialog.h>
 #include <qprinter.h>
@@ -92,6 +96,7 @@
 #include <qinputdialog.h>
 #include <qsplitter.h>
 
+#include <qurl.h>
 #include <qwebframe.h>
 #include <qwebhistory.h>
 
@@ -99,18 +104,41 @@
 
 BrowserMainWindow::BrowserMainWindow(QWidget *parent, Qt::WindowFlags flags)
     : QMainWindow(parent, flags)
+    , m_navigationBar(0)
+    , m_navigationSplitter(0)
+    , m_toolbarSearch(0)
+#if defined(Q_WS_MAC)
+    , m_bookmarksToolbarFrame(0)
+#endif
+    , m_bookmarksToolbar(0)
     , m_tabWidget(new TabWidget(this))
     , m_autoSaver(new AutoSaver(this))
+    , m_showMenuBarAction(0)
     , m_historyBack(0)
+    , m_historyBackMenu(0)
     , m_historyForward(0)
+    , m_historyForwardMenu(0)
+    , m_windowMenu(0)
+    , m_privateBrowsing(0)
     , m_stop(0)
     , m_reload(0)
+    , m_stopReload(0)
+    , m_viewToolbar(0)
+    , m_viewBookmarkBar(0)
+    , m_viewStatusbar(0)
+    , m_restoreLastSession(0)
+    , m_addBookmark(0)
 {
     setAttribute(Qt::WA_DeleteOnClose, true);
     statusBar()->setSizeGripEnabled(true);
+    // fixes https://bugzilla.mozilla.org/show_bug.cgi?id=219070
+    // yes, that's a Firefox bug!
+    statusBar()->setLayoutDirection(Qt::LeftToRight);
     setupMenu();
     setupToolBar();
 
+    m_privateBrowsing->setChecked(BrowserApplication::isPrivate());
+
     QWidget *centralWidget = new QWidget(this);
     BookmarksModel *boomarksModel = BrowserApplication::bookmarksManager()->bookmarksModel();
     m_bookmarksToolbar = new BookmarksToolBar(boomarksModel, this);
@@ -189,7 +217,9 @@ BrowserMainWindow::BrowserMainWindow(QWidget *parent, Qt::WindowFlags flags)
     loadDefaultState();
     m_tabWidget->newTab();
     m_tabWidget->currentLineEdit()->setFocus();
+#if defined(Q_WS_MAC)
     m_navigationBar->setIconSize(QSize(18, 18));
+#endif
 
     // Add each item in the menu bar to the main window so
     // if the menu bar is hidden the shortcuts still work.
@@ -202,6 +232,9 @@ BrowserMainWindow::BrowserMainWindow(QWidget *parent, Qt::WindowFlags flags)
 #if defined(Q_WS_MAC)
     setWindowIcon(QIcon());
 #endif
+#if defined(Q_WS_X11)
+    setWindowRole(QLatin1String("browser"));
+#endif
 }
 
 BrowserMainWindow::~BrowserMainWindow()
@@ -313,6 +346,8 @@ bool BrowserMainWindow::restoreState(const QByteArray &state)
     statusBar()->setVisible(showStatusbar);
     updateStatusbarActionText(showStatusbar);
 
+    m_statusBarVisible = showStatusbar;
+
     m_navigationSplitter->restoreState(splitterState);
 
     if (!tabState.isEmpty() && !tabWidget()->restoreState(tabState))
@@ -337,6 +372,9 @@ QAction *BrowserMainWindow::showMenuBarAction() const
 
 void BrowserMainWindow::setupMenu()
 {
+    m_menuBarVisible = true;
+
+    menuBar()->clear();
     new QShortcut(QKeySequence(Qt::Key_F6), this, SLOT(slotSwapFocus()));
 
     // File
@@ -445,6 +483,8 @@ void BrowserMainWindow::setupMenu()
     m_stop->setShortcuts(shortcuts);
     m_tabWidget->addWebAction(m_stop, QWebPage::Stop);
 
+    if (m_reload)
+        delete m_reload;
     m_reload = viewMenu->addAction(tr("&Reload Page"));
     m_reload->setShortcuts(QKeySequence::Refresh);
     m_tabWidget->addWebAction(m_reload, QWebPage::Reload);
@@ -524,6 +564,8 @@ void BrowserMainWindow::setupMenu()
                          QKeySequence(tr("Ctrl+K", "Web Search")));
     toolsMenu->addAction(tr("&Clear Private Data"), this, SLOT(slotClearPrivateData()),
                          QKeySequence(tr("Ctrl+Shift+Delete", "Clear Private Data")));
+    toolsMenu->addAction(tr("Show &Network Monitor"), this, SLOT(showNetworkMonitor()) );
+
 #ifndef Q_CC_MINGW
     QAction *m_enableInspector = toolsMenu->addAction(tr("Enable Web &Inspector"), this, SLOT(slotToggleInspector(bool)));
     m_enableInspector->setCheckable(true);
@@ -534,6 +576,8 @@ void BrowserMainWindow::setupMenu()
 
     // Help
     QMenu *helpMenu = menuBar()->addMenu(tr("&Help"));
+    helpMenu->addAction(tr("Switch application language "), BrowserApplication::languageManager(), SLOT(chooseNewLanguage()));
+    helpMenu->addSeparator();
     helpMenu->addAction(tr("About &Qt"), qApp, SLOT(aboutQt()));
     helpMenu->addAction(tr("About &Arora"), this, SLOT(slotAboutApplication()));
 }
@@ -566,11 +610,11 @@ void BrowserMainWindow::setupToolBar()
     m_stopReload = new QAction(this);
     m_reloadIcon = style()->standardIcon(QStyle::SP_BrowserReload);
     m_stopReload->setIcon(m_reloadIcon);
-
     m_navigationBar->addAction(m_stopReload);
 
     m_navigationSplitter = new QSplitter(m_navigationBar);
     m_navigationSplitter->addWidget(m_tabWidget->lineEditStack());
+
     m_toolbarSearch = new ToolbarSearch(m_navigationBar);
     m_navigationSplitter->addWidget(m_toolbarSearch);
     connect(m_toolbarSearch, SIGNAL(search(const QUrl&)),
@@ -596,7 +640,7 @@ void BrowserMainWindow::slotShowBookmarksDialog()
 void BrowserMainWindow::slotAddBookmark()
 {
     WebView *webView = currentTab();
-    QString url = webView->url().toString();
+    QString url = QLatin1String(webView->url().toEncoded());
     QString title = webView->title();
     AddBookmarkDialog dialog(url, title);
     dialog.exec();
@@ -605,6 +649,8 @@ void BrowserMainWindow::slotAddBookmark()
 void BrowserMainWindow::slotViewMenuBar()
 {
     menuBar()->setVisible(!menuBar()->isVisible());
+
+    m_menuBarVisible = menuBar()->isVisible();
 }
 
 void BrowserMainWindow::slotViewToolbar()
@@ -661,6 +707,9 @@ void BrowserMainWindow::slotViewStatusbar()
         updateStatusbarActionText(true);
         statusBar()->show();
     }
+
+    m_statusBarVisible = statusBar()->isVisible();
+
     m_autoSaver->changeOccurred();
 }
 
@@ -678,7 +727,7 @@ QUrl BrowserMainWindow::guessUrlFromString(const QString &string)
     bool hasSchema = test.exactMatch(urlStr);
     if (hasSchema) {
         bool isAscii = true;
-        foreach (QChar c, urlStr) {
+        foreach (const QChar &c, urlStr) {
             if (c >= 0x80) {
                 isAscii = false;
                 break;
@@ -815,9 +864,7 @@ void BrowserMainWindow::printRequested(QWebFrame *frame)
 
 void BrowserMainWindow::slotPrivateBrowsing()
 {
-    QWebSettings *settings = QWebSettings::globalSettings();
-    bool pb = settings->testAttribute(QWebSettings::PrivateBrowsingEnabled);
-    if (!pb) {
+    if (!BrowserApplication::isPrivate()) {
         QString title = tr("Are you sure you want to turn on private browsing?");
         QString text = tr("<b>%1</b><br><br>When private browsing is turned on,"
             " some actions concerning your privacy will be disabled:"
@@ -833,18 +880,20 @@ void BrowserMainWindow::slotPrivateBrowsing()
                                QMessageBox::Ok | QMessageBox::Cancel,
                                QMessageBox::Ok);
         if (button == QMessageBox::Ok) {
-            settings->setAttribute(QWebSettings::PrivateBrowsingEnabled, true);
+            BrowserApplication::setPrivate(true);
+        } else {
+            m_privateBrowsing->setChecked(false);
         }
     } else {
-        settings->setAttribute(QWebSettings::PrivateBrowsingEnabled, false);
-
-        QList<BrowserMainWindow*> windows = BrowserApplication::instance()->mainWindows();
-        for (int i = 0; i < windows.count(); ++i) {
-            BrowserMainWindow *window = windows.at(i);
-            window->tabWidget()->clear();
-        }
+        BrowserApplication::setPrivate(false);
     }
-    m_privateBrowsing->setChecked(settings->testAttribute(QWebSettings::PrivateBrowsingEnabled));
+}
+
+void BrowserMainWindow::slotPrivacyChanged(bool isPrivate)
+{
+    m_privateBrowsing->setChecked(isPrivate);
+    if (!isPrivate)
+        tabWidget()->clear();
 }
 
 void BrowserMainWindow::closeEvent(QCloseEvent *event)
@@ -871,18 +920,24 @@ void BrowserMainWindow::closeEvent(QCloseEvent *event)
 
 void BrowserMainWindow::mousePressEvent(QMouseEvent *event)
 {
-	switch(event->button())
-	{
-	case Qt::XButton1:
-		m_historyBack->activate(QAction::Trigger);
-		break;
-	case Qt::XButton2:
-		m_historyForward->activate(QAction::Trigger);
-		break;
-	default:
-		QMainWindow::mousePressEvent(event);
-		break;
-	}
+    switch(event->button()) {
+    case Qt::XButton1:
+        m_historyBack->activate(QAction::Trigger);
+        break;
+    case Qt::XButton2:
+        m_historyForward->activate(QAction::Trigger);
+        break;
+    default:
+        QMainWindow::mousePressEvent(event);
+        break;
+    }
+}
+
+void BrowserMainWindow::changeEvent(QEvent *event)
+{
+    if (event->type() == QEvent::LanguageChange)
+        retranslate();
+    QMainWindow::changeEvent(event);
 }
 
 void BrowserMainWindow::slotEditFind()
@@ -925,8 +980,14 @@ void BrowserMainWindow::slotViewFullScreen(bool makeFullScreen)
 {
     if (makeFullScreen) {
         setWindowState(windowState() | Qt::WindowFullScreen);
+
+        menuBar()->hide();
+        statusBar()->hide();
     } else {
         setWindowState(windowState() & ~Qt::WindowFullScreen);
+
+        menuBar()->setVisible(m_menuBarVisible);
+        statusBar()->setVisible(m_statusBarVisible);
     }
 }
 
@@ -935,12 +996,12 @@ void BrowserMainWindow::slotViewPageSource()
     if (!currentTab())
         return;
 
+    QString title = currentTab()->title();
     QString markup = currentTab()->page()->mainFrame()->toHtml();
-    QPlainTextEdit *view = new QPlainTextEdit(markup);
-    view->setWindowTitle(tr("Page Source of %1").arg(currentTab()->title()));
-    view->setMinimumWidth(640);
-    view->setAttribute(Qt::WA_DeleteOnClose);
-    view->show();
+    QUrl url = currentTab()->url();
+    SourceViewer *viewer = new SourceViewer(markup, title, url, this);
+    viewer->setAttribute(Qt::WA_DeleteOnClose);
+    viewer->show();
 }
 
 void BrowserMainWindow::slotHome()
@@ -951,6 +1012,12 @@ void BrowserMainWindow::slotHome()
     loadPage(home);
 }
 
+void BrowserMainWindow::retranslate()
+{
+    setupMenu();
+    m_navigationBar->setWindowTitle(tr("Navigation"));
+}
+
 void BrowserMainWindow::slotWebSearch()
 {
     m_toolbarSearch->selectAll();
@@ -1116,3 +1183,9 @@ void BrowserMainWindow::geometryChangeRequested(const QRect &geometry)
     setGeometry(geometry);
 }
 
+void BrowserMainWindow::showNetworkMonitor()
+{
+    NetworkMonitor *monitor = NetworkMonitor::self();
+    monitor->show();
+}
+
