@@ -103,10 +103,13 @@ public:
 public slots:
     void loadPage(const QString &url);
     void slotHome();
+    void slotPrivacyChanged(bool isPrivate);
 
 protected:
     void closeEvent(QCloseEvent *event);
     void mousePressEvent(QMouseEvent *event);
+    void changeEvent(QEvent *event);
+
 private slots:
     void save();
 
@@ -155,8 +158,10 @@ private slots:
     void geometryChangeRequested(const QRect &geometry);
     void updateToolbarActionText(bool visible);
     void updateBookmarksToolbarActionText(bool visible);
+    void showNetworkMonitor();
 
 private:
+    void retranslate();
     void loadDefaultState();
     void setupMenu();
     void setupToolBar();
@@ -190,6 +195,9 @@ private:
     QAction *m_restoreLastSession;
     QAction *m_addBookmark;
 
+    bool m_menuBarVisible;
+    bool m_statusBarVisible;
+
     QIcon m_reloadIcon;
     QIcon m_stopIcon;
 };
