@@ -90,6 +90,8 @@ DownloadItem::DownloadItem(QNetworkReply *reply, bool requestFileName, QWidget *
     , m_reply(reply)
     , m_requestFileName(requestFileName)
     , m_bytesReceived(0)
+    , m_startedSaving(false)
+    , m_finishedDownloading(false)
     , m_gettingFileName(false)
 {
     setupUi(this);
@@ -340,11 +342,11 @@ void DownloadItem::updateInfoLabel()
             if (timeRemaining > 60) {
                 timeRemaining = timeRemaining / 60;
                 timeRemaining = floor(timeRemaining);
-                remaining = tr("- %n minutes remaining", "", timeRemaining);
+                remaining = tr("- %n minutes remaining", "", int(timeRemaining));
             }
             else {
                 timeRemaining = floor(timeRemaining);
-                remaining = tr("- %n seconds remaining", "", timeRemaining);
+                remaining = tr("- %n seconds remaining", "", int(timeRemaining));
             }
         }
         info = QString(tr("%1 of %2 (%3/sec) %4"))
@@ -414,6 +416,7 @@ void DownloadItem::finished()
 DownloadManager::DownloadManager(QWidget *parent)
     : QDialog(parent)
     , m_autoSaver(new AutoSaver(this))
+    , m_model(new DownloadModel(this))
     , m_manager(BrowserApplication::networkAccessManager())
     , m_iconProvider(0)
     , m_removePolicy(Never)
@@ -424,7 +427,6 @@ DownloadManager::DownloadManager(QWidget *parent)
     downloadsView->horizontalHeader()->hide();
     downloadsView->setAlternatingRowColors(true);
     downloadsView->horizontalHeader()->setStretchLastSection(true);
-    m_model = new DownloadModel(this);
     downloadsView->setModel(m_model);
     connect(cleanupButton, SIGNAL(clicked()), this, SLOT(cleanup()));
     connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
