@@ -26,6 +26,7 @@
 #include <qurl.h>
 
 class WebView;
+class QLabel;
 class LocationBarSiteIcon;
 class LocationBar : public LineEdit
 {
@@ -35,9 +36,13 @@ public:
     LocationBar(QWidget *parent = 0);
     void setWebView(WebView *webView);
 
+public slots:
+    void setPrivate(bool isPrivate);
+
 protected:
     void paintEvent(QPaintEvent *event);
     void focusOutEvent(QFocusEvent *event);
+    void mouseDoubleClickEvent(QMouseEvent *event);
 
 private slots:
     void webViewUrlChanged(const QUrl &url);
@@ -46,6 +51,7 @@ private:
     QPointer<WebView> m_webView;
     LocationBarSiteIcon *m_siteIcon;
     QColor m_defaultBaseColor;
+    QLabel *m_privacyIndicator;
 };
 
 #endif // LOCATIONBAR_H
