@@ -195,7 +195,7 @@ SearchLineEdit::SearchLineEdit(QWidget *parent)
     m_searchButton = new SearchButton(this);
     updateGeometries();
     addWidget(m_searchButton, LeftSide);
-    m_inactiveText = tr("Search");
+    setInactiveText(tr("Search"));
 
     QSizePolicy policy = sizePolicy();
     setSizePolicy(QSizePolicy::Preferred, policy.verticalPolicy());
@@ -212,24 +212,6 @@ SearchLineEdit::SearchLineEdit(QWidget *parent)
     setUpdatesEnabled(true);
 }
 
-void SearchLineEdit::paintEvent(QPaintEvent *event)
-{
-    if (text().isEmpty() && !hasFocus() && !m_inactiveText.isEmpty()) {
-        LineEdit::paintEvent(event);
-        QStyleOptionFrameV2 panel;
-        initStyleOption(&panel);
-        QRect textRect = style()->subElementRect(QStyle::SE_LineEditContents, &panel, this);
-        int left = textMargin(LineEdit::LeftSide);
-        int right = textMargin(LineEdit::RightSide);
-        textRect.adjust(left, 0, -right, 0);
-        QPainter painter(this);
-        painter.setPen(palette().brush(QPalette::Disabled, QPalette::Text).color());
-        painter.drawText(textRect, Qt::AlignLeft | Qt::AlignVCenter, m_inactiveText);
-    } else {
-        LineEdit::paintEvent(event);
-    }
-}
-
 void SearchLineEdit::resizeEvent(QResizeEvent *event)
 {
     updateGeometries();
@@ -247,16 +229,6 @@ void SearchLineEdit::updateGeometries()
     updateTextMargins();
 }
 
-QString SearchLineEdit::inactiveText() const
-{
-    return m_inactiveText;
-}
-
-void SearchLineEdit::setInactiveText(const QString &text)
-{
-    m_inactiveText = text;
-}
-
 void SearchLineEdit::setMenu(QMenu *menu)
 {
     if (m_searchButton->m_menu)
