@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Benjamin C. Meyer <ben@meyerhome.net>
+ * Copyright 2008-2009 Benjamin C. Meyer <ben@meyerhome.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,6 +64,8 @@
 
 #include "browserapplication.h"
 #include "browsermainwindow.h"
+#include "cookiedialog.h"
+#include "cookieexceptionsdialog.h"
 #include "cookiejar.h"
 #include "history.h"
 #include "networkaccessmanager.h"
@@ -85,6 +87,9 @@ SettingsDialog::SettingsDialog(QWidget *parent)
     connect(standardFontButton, SIGNAL(clicked()), this, SLOT(chooseFont()));
     connect(fixedFontButton, SIGNAL(clicked()), this, SLOT(chooseFixedFont()));
 
+#if QT_VERSION < 0x040500
+    oneCloseButton->setVisible(false); // no other mode than one close button with qt <4.5
+#endif
     loadDefaults();
     loadFromSettings();
 }
@@ -216,6 +221,9 @@ void SettingsDialog::loadFromSettings()
     settings.beginGroup(QLatin1String("tabs"));
     selectTabsWhenCreated->setChecked(settings.value(QLatin1String("selectNewTabs"), false).toBool());
     confirmClosingMultipleTabs->setChecked(settings.value(QLatin1String("confirmClosingMultipleTabs"), true).toBool());
+#if QT_VERSION >= 0x040500
+    oneCloseButton->setChecked(settings.value(QLatin1String("oneCloseButton"),false).toBool());
+#endif
     settings.endGroup();
 }
 
@@ -318,6 +326,9 @@ void SettingsDialog::saveToSettings()
     settings.beginGroup(QLatin1String("tabs"));
     settings.setValue(QLatin1String("selectNewTabs"), selectTabsWhenCreated->isChecked());
     settings.setValue(QLatin1String("confirmClosingMultipleTabs"), confirmClosingMultipleTabs->isChecked());
+#if QT_VERSION >= 0x040500
+    settings.setValue(QLatin1String("oneCloseButton"), oneCloseButton->isChecked());
+#endif
     settings.endGroup();
 
     BrowserApplication::instance()->loadSettings();
@@ -334,13 +345,13 @@ void SettingsDialog::accept()
 
 void SettingsDialog::showCookies()
 {
-    CookiesDialog *dialog = new CookiesDialog(BrowserApplication::cookieJar(), this);
+    CookieDialog *dialog = new CookieDialog(BrowserApplication::cookieJar(), this);
     dialog->exec();
 }
 
 void SettingsDialog::showExceptions()
 {
-    CookiesExceptionsDialog *dialog = new CookiesExceptionsDialog(BrowserApplication::cookieJar(), this);
+    CookieExceptionsDialog *dialog = new CookieExceptionsDialog(BrowserApplication::cookieJar(), this);
     dialog->exec();
 }
 
