@@ -128,19 +128,27 @@ TabWidget::TabWidget(QWidget *parent)
     m_closeTabAction->setShortcuts(QKeySequence::Close);
     connect(m_closeTabAction, SIGNAL(triggered()), this, SLOT(closeTab()));
 
-#if QT_VERSION < 0x040500
-    m_newTabAction->setIcon(QIcon(QLatin1String(":addtab.png")));
-    m_newTabAction->setIconVisibleInMenu(false);
+#if QT_VERSION >= 0x040500
+    QSettings settings;
+    settings.beginGroup(QLatin1String("tabs"));
+    bool oneCloseButton = settings.value(QLatin1String("oneCloseButton"), false).toBool();
+    if (oneCloseButton) {
+#endif
+        // With Qt < 4.5 do this always, with >=4.5 only if enabled.
+        m_newTabAction->setIcon(QIcon(QLatin1String(":addtab.png")));
+        m_newTabAction->setIconVisibleInMenu(false);
 
-    m_closeTabAction->setIcon(QIcon(QLatin1String(":closetab.png")));
-    m_closeTabAction->setIconVisibleInMenu(false);
+        m_closeTabAction->setIcon(QIcon(QLatin1String(":closetab.png")));
+        m_closeTabAction->setIconVisibleInMenu(false);
+#if QT_VERSION >= 0x040500
+    }
 #endif
 
     m_nextTabAction = new QAction(tr("Show Next Tab"), this);
     QList<QKeySequence> shortcuts;
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_BraceRight));
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_PageDown));
-    shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_BracketRight));
+    shortcuts.append(tr("Ctrl-]"));
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_Less));
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_Tab));
     m_nextTabAction->setShortcuts(shortcuts);
@@ -150,7 +158,7 @@ TabWidget::TabWidget(QWidget *parent)
     shortcuts.clear();
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_BraceLeft));
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_PageUp));
-    shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_BracketLeft));
+    shortcuts.append(tr("Ctrl-["));
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::Key_Greater));
     shortcuts.append(QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_Tab));
     m_previousTabAction->setShortcuts(shortcuts);
@@ -166,35 +174,51 @@ TabWidget::TabWidget(QWidget *parent)
     m_recentlyClosedTabsAction->setEnabled(false);
 
 #if QT_VERSION >= 0x040500
-    m_tabBar->setTabsClosable(true);
-    connect(m_tabBar, SIGNAL(tabCloseRequested(int)),
-            this, SLOT(closeTab(int)));
-    m_tabBar->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
-#else
-    // corner buttons
-    QToolButton *addTabButton = new QToolButton(this);
-    addTabButton->setDefaultAction(m_newTabAction);
-    addTabButton->setAutoRaise(true);
-    addTabButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
-    setCornerWidget(addTabButton, Qt::TopLeftCorner);
+    if (oneCloseButton) {
+#endif
+        // corner buttons
+        // With Qt < 4.5 do this always, with >=4.5 only if enabled.
+        QToolButton *addTabButton = new QToolButton(this);
+        addTabButton->setDefaultAction(m_newTabAction);
+        addTabButton->setAutoRaise(true);
+        addTabButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
+        setCornerWidget(addTabButton, Qt::TopLeftCorner);
 
-    QToolButton *closeTabButton = new QToolButton(this);
-    closeTabButton->setDefaultAction(m_closeTabAction);
-    closeTabButton->setAutoRaise(true);
-    closeTabButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
-    setCornerWidget(closeTabButton, Qt::TopRightCorner);
+        QToolButton *closeTabButton = new QToolButton(this);
+        closeTabButton->setDefaultAction(m_closeTabAction);
+        closeTabButton->setAutoRaise(true);
+        closeTabButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
+        setCornerWidget(closeTabButton, Qt::TopRightCorner);
+#if QT_VERSION >= 0x040500
+    } else {
+        m_tabBar->setTabsClosable(true);
+
+        connect(m_tabBar, SIGNAL(tabCloseRequested(int)),
+                this, SLOT(closeTab(int)));
+        m_tabBar->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
+    }
 #endif
 
     connect(this, SIGNAL(currentChanged(int)),
             this, SLOT(currentChanged(int)));
 
     m_lineEdits = new QStackedWidget(this);
+
+    connect(BrowserApplication::historyManager(), SIGNAL(historyCleared()),
+        this, SLOT(historyCleared()));
+}
+
+void TabWidget::historyCleared()
+{
+    m_recentlyClosedTabs.clear();
+    m_recentlyClosedTabsAction->setEnabled(false);
 }
 
 void TabWidget::clear()
 {
     // clear the recently closed tabs
     m_recentlyClosedTabs.clear();
+    m_recentlyClosedTabsAction->setEnabled(false);
     // clear the line edit history
     for (int i = 0; i < m_lineEdits->count(); ++i) {
         QLineEdit *qLineEdit = lineEdit(i);
@@ -211,9 +235,9 @@ void TabWidget::reloadTab(int index)
     if (index < 0 || index >= count())
         return;
 
-    QWidget *widget = this->widget(index);
-    if (WebView *tab = qobject_cast<WebView*>(widget))
+    if (WebView *tab = webView(index)) {
         tab->reload();
+    }
 }
 
 void TabWidget::moveTab(int fromIndex, int toIndex)
@@ -414,10 +438,11 @@ WebView *TabWidget::makeNewTab(bool makeCurrent)
 
     // line edit
     LocationBar *locationBar = new LocationBar;
-    if (!m_lineEditCompleter && count() > 0) {
+    if (!m_lineEditCompleter) {
         HistoryCompletionModel *completionModel = new HistoryCompletionModel(this);
         completionModel->setSourceModel(BrowserApplication::historyManager()->historyFilterModel());
         m_lineEditCompleter = new QCompleter(completionModel, this);
+        connect(m_lineEditCompleter, SIGNAL(activated(QString)), mainWindow(), SLOT(loadPage(QString)));
         // Should this be in Qt by default?
         QAbstractItemView *popup = m_lineEditCompleter->popup();
         QListView *listView = qobject_cast<QListView*>(popup);
@@ -519,8 +544,7 @@ void TabWidget::toolBarVisibilityChangeRequestedCheck(bool visible)
 void TabWidget::reloadAllTabs()
 {
     for (int i = 0; i < count(); ++i) {
-        QWidget *tabWidget = widget(i);
-        if (WebView *tab = qobject_cast<WebView*>(tabWidget)) {
+        if (WebView *tab = webView(i)) {
             tab->reload();
         }
     }
@@ -578,7 +602,9 @@ void TabWidget::closeTab(int index)
         return;
 
     bool hasFocus = false;
-    if (WebView *tab = webView(index)) {
+    WebView *tab = webView(index);
+
+    if (tab && !tab->url().isEmpty()) {
         if (tab->isModified()) {
             QMessageBox closeConfirmation(tab);
             closeConfirmation.setWindowFlags(Qt::Sheet);
@@ -709,7 +735,7 @@ void TabWidget::webViewTitleChanged(const QString &title)
     }
     if (currentIndex() == index)
         emit setCurrentTitle(title);
-    BrowserApplication::historyManager()->updateHistoryItem(webView->url(), title);
+    BrowserApplication::historyManager()->updateHistoryEntry(webView->url(), title);
 }
 
 void TabWidget::webViewUrlChanged(const QUrl &url)
