@@ -111,7 +111,7 @@ bool WebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &r
     // ctrl open in new tab
     // ctrl-shift open in new tab and select
     // ctrl-alt open in new window
-    if (type == QWebPage::NavigationTypeLinkClicked
+    if ((type == QWebPage::NavigationTypeLinkClicked || type == QWebPage::NavigationTypeOther)
         && (m_keyboardModifiers & Qt::ControlModifier
             || m_pressedButtons == Qt::MidButton)) {
         bool newWindow = (m_keyboardModifiers & Qt::AltModifier);
@@ -191,13 +191,18 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
     }
 
     QFile file(QLatin1String(":/notfound.html"));
-    bool isOpened = file.open(QIODevice::ReadOnly);
-    Q_ASSERT(isOpened);
+    if (!file.open(QIODevice::ReadOnly)) {
+        qWarning() << "WebPage::handleUnsupportedContent" << "Unable to open notfound.html";
+        return;
+    }
     QString title = tr("Error loading page: %1").arg(reply->url().toString());
     QString html = QString(QLatin1String(file.readAll()))
                         .arg(title)
                         .arg(reply->errorString())
-                        .arg(reply->url().toString());
+                        .arg(tr("When connecting to: %1.").arg(reply->url().toString()))
+                        .arg(tr("Check the address for errors such as <b>ww</b>.arora-browser.org instead of <b>www</b>.arora-browser.org"))
+                        .arg(tr("If the address is correct, try to check the network connection."))
+                        .arg(tr("If your computer or network is protected by a firewall or proxy, make sure that the browser is permitted to access the network."));
 
     QBuffer imageBuffer;
     imageBuffer.open(QBuffer::ReadWrite);
@@ -226,7 +231,7 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 }
 
 
-WebView::WebView(QWidget* parent)
+WebView::WebView(QWidget *parent)
     : QWebView(parent)
     , m_progress(0)
     , m_currentZoom(100)
@@ -244,6 +249,7 @@ WebView::WebView(QWidget* parent)
     connect(page(), SIGNAL(downloadRequested(const QNetworkRequest &)),
             this, SLOT(downloadRequested(const QNetworkRequest &)));
     page()->setForwardUnsupportedContent(true);
+    setAcceptDrops(true);
 
     // the zoom values (in percent) are chosen to be like in Mozilla Firefox 3
     m_zoomLevels << 30 << 50 << 67 << 80 << 90;
@@ -264,6 +270,8 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         menu->addAction(tr("Save Lin&k"), this, SLOT(downloadLinkToDisk()));
         menu->addAction(tr("&Bookmark This Link"), this, SLOT(bookmarkLink()))->setData(r.linkUrl().toString());
         menu->addSeparator();
+        if (!page()->selectedText().isEmpty())
+            menu->addAction(pageAction(QWebPage::Copy));
         menu->addAction(tr("&Copy Link Location"), this, SLOT(copyLinkToClipboard()));
         if (page()->settings()->testAttribute(QWebSettings::DeveloperExtrasEnabled))
             menu->addAction(pageAction(QWebPage::InspectElement));
@@ -280,9 +288,11 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         menu->addAction(tr("C&opy Image Location"), this, SLOT(copyImageLocationToClipboard()))->setData(r.imageUrl().toString());
     }
 
-#ifdef WEBKIT_TRUNK // i.e. Qt 4.5, but not in Qt 4.5 yet
-    if (menu->isEmpty())
+#if QT_VERSION >= 0x040500
+    if (menu->isEmpty()) {
+        delete menu;
         menu = page()->createStandardContextMenu();
+    }
 #endif
 
     if (!menu->isEmpty()) {
@@ -469,6 +479,42 @@ void WebView::mousePressEvent(QMouseEvent *event)
     QWebView::mousePressEvent(event);
 }
 
+void WebView::dragEnterEvent(QDragEnterEvent *event)
+{
+    event->acceptProposedAction();
+}
+
+void WebView::dragMoveEvent(QDragMoveEvent *event)
+{
+    if (!event->mimeData()->urls().isEmpty()) {
+        event->acceptProposedAction();
+    } else {
+        QUrl url(event->mimeData()->text());
+        if (url.isValid())
+            event->acceptProposedAction();
+    }
+    if (!event->isAccepted())
+        QWebView::dragMoveEvent(event);
+}
+
+void WebView::dropEvent(QDropEvent *event)
+{
+    QWebView::dropEvent(event);
+    if (!event->isAccepted()
+        && event->possibleActions() & Qt::CopyAction) {
+
+        QUrl url;
+        if (!event->mimeData()->urls().isEmpty())
+            url = event->mimeData()->urls().first();
+        if (!url.isValid())
+            url = event->mimeData()->text();
+        if (url.isValid()) {
+            loadUrl(url);
+            event->acceptProposedAction();
+        }
+    }
+}
+
 void WebView::mouseReleaseEvent(QMouseEvent *event)
 {
     QWebView::mouseReleaseEvent(event);
